from .Singleton import Singleton
from brain.core.Models.settings.Resources import Resources
from .Brain import Brain
from .Synapse import Synapse
from .Neuron import Neuron
from .Signal import Signal



import inspect
import logging
import os
from six import with_metaclass
import six

from brain.core.Models.Signal import Signal
from .YAMLLoader import YAMLLoader
from brain.core.Utils import Utils
from brain.core.ConfigurationManager import SettingLoader
from brain.core.ConfigurationManager.ConfigurationChecker import ConfigurationChecker
from brain.core.Models import Singleton
from brain.core.Models.Brain import Brain
from brain.core.Models.Neuron import Neuron
from brain.core.Models.Synapse import Synapse

logging.basicConfig()
logger = logging.getLogger("brain")

FILE_NAME = "brain.yml"


class BrainNotFound(Exception):
    pass


class BrainLoader(with_metaclass(Singleton, object)):
    """
    This Class is used to get the brain YAML and the Brain as an object
    """

    def __init__(self, file_path=None):
        sl = SettingLoader()
        self.settings = sl.settings

        self.file_path = file_path
        if self.file_path is None:  # we don't provide a file path, so search for the default one
            self.file_path = Utils.get_real_file_path(FILE_NAME)
        else:
            self.file_path = Utils.get_real_file_path(file_path)
        # if the returned file path is none, the file doesn't exist
        if self.file_path is None:
            raise BrainNotFound("brain file not found")
        self.yaml_config = self.get_yaml_config()
        self.brain = self.load_brain()

    def get_yaml_config(self):
        """
        Class Methods which loads default or the provided YAML file and return it as a String
        :return: The loaded brain YAML
        :rtype: String

        :Example:
            brain_yaml = BrainLoader.get_yaml_config(/var/tmp/brain.yml)

        .. warnings:: Class Method
        """
        if self.file_path is None:
            brain_file_path = self._get_root_brain_path()
        else:
            brain_file_path = self.file_path
        return YAMLLoader.get_config(brain_file_path)

    def load_brain(self):
        """
        Class Methods which loads default or the provided YAML file and return a Brain
        :return: The loaded Brain
        :rtype: Brain

        :Example:

            brain = BrainLoader.load_brain(file_path="/var/tmp/brain.yml")

        .. seealso:: Brain
        .. warnings:: Class Method
        """

        # Instantiate a brain
        brain = Brain()

        # get the brain with dict
        dict_brain = self.get_yaml_config()

        brain.brain_yaml = dict_brain
        # create list of Synapse
        synapses = list()
        for synapses_dict in dict_brain:
            if "includes" not in synapses_dict:     # we don't need to check includes as it's not a synapse
                if ConfigurationChecker().check_synape_dict(synapses_dict):
                    name = synapses_dict["name"]
                    neurons = self._get_neurons(synapses_dict["neurons"], self.settings)
                    signals = self._get_signals(synapses_dict["signals"])
                    new_synapse = Synapse(name=name, neurons=neurons, signals=signals)
                    synapses.append(new_synapse)
        brain.synapses = synapses
        if self.file_path is None:
            brain.brain_file = self._get_root_brain_path()
        else:
            brain.brain_file = self.file_path
        # check that no synapse have the same name than another
        if not ConfigurationChecker().check_synapes(synapses):
            brain = None

        return brain

    @classmethod
    def _get_neurons(cls, neurons_dict, settings):
        """
        Get a list of Neuron object from a neuron dict

        :param neurons_dict: Neuron name or dictionary of Neuron_name/Neuron_parameters
        :type neurons_dict: String or dict
        :param settings:  The Settings with the global variables
        :return: A list of Neurons
        :rtype: List

        :Example:

            neurons = cls._get_neurons(synapses_dict["neurons"])

        .. seealso:: Neuron
        .. warnings:: Static and Private
        """

        neurons = list()
        for neuron_dict in neurons_dict:
            if ConfigurationChecker().check_neuron_dict(neuron_dict):
                if isinstance(neuron_dict, dict):
                    for neuron_name in neuron_dict:
                        new_neuron = Neuron(name=neuron_name, parameters=neuron_dict[neuron_name])
                        neurons.append(new_neuron)
                else:
                    new_neuron = Neuron(name=neuron_dict)
                    neurons.append(new_neuron)
        return neurons

    @classmethod
    def _get_signals(cls, signals_dict):
        """
        Get a list of Signal object from a signals dict

        :param signals_dict: Signal name or dictionary of Signal_name/Signal_parameters
        :type signals_dict: String or dict
        :return: A list of Event and/or Order
        :rtype: List

        :Example:

            signals = cls._get_signals(synapses_dict["signals"])

        .. seealso:: Event, Order
        .. warnings:: Class method and Private
        """
        signals = list()
        for signal_dict in signals_dict:
            if ConfigurationChecker().check_signal_dict(signal_dict):
                for signal_name in signal_dict:
                    new_signal = Signal(name=signal_name, parameters=signal_dict[signal_name])
                    signals.append(new_signal)

        return signals

    @staticmethod
    def _get_root_brain_path():
        """
        Return the full path of the default brain file

        :Example:

            brain.brain_file = cls._get_root_brain_path()

        .. raises:: IOError
        .. warnings:: Static method and Private
        """

        # get current script directory path. We are in /an/unknown/path/brain/core/ConfigurationManager
        cur_script_directory = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
        # get parent dir. Now we are in /an/unknown/path/brain
        parent_dir = os.path.normpath(cur_script_directory + os.sep + os.pardir + os.sep + os.pardir)
        brain_path = parent_dir + os.sep + "brain.yml"
        logger.debug("Real brain.yml path: %s" % brain_path)
        if os.path.isfile(brain_path):
            return brain_path
        raise IOError("Default brain.yml file not found")



from brain.core.Models.settings.SettingsEntry import SettingsEntry


class Tts(SettingsEntry):
    """
    This Class is representing a Text To Speech (TTS) with its name and parameters

    .. note:: must be defined in the settings.yml
    """

    def __init__(self, name=None, parameters=None):
        super(Tts, self).__init__(name=name)
        self.parameters = parameters

    def __str__(self):
        return str(self.serialize())

    def serialize(self):
        return {
            'name': self.name,
            'parameters': self.parameters
        }

    def __eq__(self, other):
        """
        This is used to compare 2 objects
        :param other: the Tts to compare
        :return: True if both ttss are similar, False otherwise
        """
        return self.__dict__ == other.__dict__



from brain.core.Models.settings.SettingsEntry import SettingsEntry


class Resources(SettingsEntry):
    """

    """
    def __init__(self, neuron_folder=None, stt_folder=None, tts_folder=None, trigger_folder=None, signal_folder=None):
        super(Resources, self).__init__("Resources")
        self.neuron_folder = neuron_folder
        self.stt_folder = stt_folder
        self.tts_folder = tts_folder
        self.trigger_folder = trigger_folder
        self.signal_folder = signal_folder

    def __str__(self):
        return str(self.serialize())

    def serialize(self):
        """
        This method allows to serialize in a proper way this object

        :return: A dict of order
        :rtype: Dict
        """

        return {
            'neuron_folder': self.neuron_folder,
            'stt_folder': self.stt_folder,
            'tts_folder': self.tts_folder,
            'trigger_folder': self.trigger_folder,
            'signal_folder': self.signal_folder
        }

    def __eq__(self, other):
        """
        This is used to compare 2 objects
        :param other:
        :return:
        """
        return self.__dict__ == other.__dict__



import logging

from brain.core.Utils import Utils
from brain.core.HookManager import HookManager
from brain.core.ConfigurationManager import SettingLoader

logging.basicConfig()
logger = logging.getLogger("brain")


class SettingEditor(object):
    """This Static class provides methods/functions to update properties from the Settings"""

    @staticmethod
    def _check_name_in_list_settings_entry(name_to_check, list_settings_entry):
        """
        manage object models : STT, TRIGGERS, TTS, PLAYERS because they have "name" attributes
        :param name_to_check: name to find in the list_settings_entry ~brain.core.Models.settings.SettingsEntry.SettingsEntry
        :param list_settings_entry: the list of SettingsEntry to inspect
        :return: True if the name_to_check corresponds to a name in the SettingsEntry list provided.
        """
        found = False
        for settings_entry in list_settings_entry:
            if settings_entry.name == name_to_check:
                found = True
                break
        return found

    # Options
    @staticmethod
    def set_mute_status(mute=False):
        """
        Define is the mute status
        :param mute: Boolean. If false, Brain.ai is voice is stopped
        """
        logger.debug("[SettingEditor] mute. Switch trigger process to mute : %s" % mute)
        settings = SettingLoader().settings
        if mute:
            Utils.print_info("Brain.ai now muted, voice has been stopped.")
            HookManager.on_mute()
        else:
            Utils.print_info("Brain.ai now speaking.")
            HookManager.on_unmute()
        settings.options.mute = mute

    @staticmethod
    def set_deaf_status(trigger_instance, deaf=False):
        """
        Define is the trigger is listening or not.
        :param trigger_instance: the trigger instance coming from the order. It will be paused or unpaused.
        :param deaf: Boolean. If true, brain is trigger is paused
        """
        logger.debug("[MainController] deaf . Switch trigger process to deaf : %s" % deaf)
        settings = SettingLoader().settings
        if deaf:
            trigger_instance.pause()
            Utils.print_info("Brain.ai now deaf, trigger has been paused")
            HookManager.on_deaf()
        else:
            trigger_instance.unpause()
            Utils.print_info("Brain.ai now listening for trigger detection")
            HookManager.on_undeaf()
        settings.options.deaf = deaf

    @staticmethod
    def set_adjust_for_ambient_noise_second(adjust_for_ambient_noise_second):
        """
        Set a new value for the adjust_for_ambient_noise_second;
        Must be an integer.
        :param adjust_for_ambient_noise_second: new value to push to the adjust_for_ambient_noise_second in the Options settings
        """
        if isinstance(adjust_for_ambient_noise_second, int):
            settings = SettingLoader().settings
            settings.options.adjust_for_ambient_noise_second = adjust_for_ambient_noise_second

    @staticmethod
    def set_energy_threshold(energy_threshold):
        """
        Set the new value of the energy threshold to the settings.
        Must be an integer.
        :param energy_threshold: new value for the energy_threshold to push into the settings
        """
        if isinstance(energy_threshold, int):
            settings = SettingLoader().settings
            settings.options.energy_threshold = energy_threshold

    # Players
    @classmethod
    def set_default_player(cls, default_player_name):
        """
        Set dynamically a new default_player in the settings
        :param default_player_name: string value
        """
        settings = SettingLoader().settings
        if cls._check_name_in_list_settings_entry(default_player_name, settings.players):
            settings.default_player_name = default_player_name
        else:
            logger.debug("[Settings] default_player %s is not defined in settings file ", default_player_name)

    @staticmethod
    def set_players(new_player):
        """
        Add a new Player object in the list of players in the settings.
        If PLayer already exists in settings, it will be updated with the new player provided values.
        :param new_player: the new PLayer object to add in the settings.
        """
        settings = SettingLoader().settings
        list_no_duplicate_player = [player for player in settings.players if player.name != new_player.name]
        list_no_duplicate_player.append(new_player)
        settings.players = list_no_duplicate_player

    # TTS
    @classmethod
    def set_default_tts(cls, default_tts_name):
        """
        Set dynamically a new default_tts_name in the settings
        :param default_tts_name: string value
        """
        settings = SettingLoader().settings
        # Verify that the default name exists in the settings list
        if cls._check_name_in_list_settings_entry(default_tts_name, settings.ttss):
            settings.default_tts_name = default_tts_name
        else:
            logger.debug("[SettingsEditor] default_tts %s is not defined in settings file ", default_tts_name)

    @staticmethod
    def set_ttss(new_tts):
        """
        Add a new TTS object in the list of tts in the settings.
        If TTS already exists in settings, it will be updated with the new tts provided values.
        :param new_tts: the new TTS object to add in the settings.
        """
        settings = SettingLoader().settings
        list_no_duplicate_tts = [tts for tts in settings.ttss if tts.name != new_tts.name]
        list_no_duplicate_tts.append(new_tts)
        settings.ttss = list_no_duplicate_tts

    # STT
    @classmethod
    def set_default_stt(cls, default_stt_name):
        """
        Set dynamically a new default_stt_name in the settings if in the list of stts.
        :param default_stt_name: string value
        """
        settings = SettingLoader().settings
        if cls._check_name_in_list_settings_entry(default_stt_name, settings.stts):
            settings.default_stt_name = default_stt_name
        else:
            logger.debug("[Settings] default_stt %s is not defined in settings file ", default_stt_name)

    @staticmethod
    def set_stts(new_stt):
        """
        Add or update the speak to text list defined in the settings.
        :param new_stt: The new stt instance.
        """
        settings = SettingLoader().settings
        list_no_duplicate_stt = [stt for stt in settings.stts if stt.name != new_stt.name]
        list_no_duplicate_stt.append(new_stt)
        settings.stts = list_no_duplicate_stt

    # TRIGGER
    @classmethod
    def set_default_trigger(cls, default_trigger):
        """
        Set dynamically a new default_trigger in the settingss
        :param default_trigger: string value
        """
        settings = SettingLoader().settings
        if cls._check_name_in_list_settings_entry(default_trigger, settings.triggers):
            settings.default_trigger_name = default_trigger
        else:
            logger.debug("[Settings] default_trigger %s is not defined in settings file ", default_trigger)

    @staticmethod
    def set_trigger(new_trigger):
        """
        Update the list of triggers with a new trigger instance.
        If the trigger name already exists then it will be updated otherwise it will be added.
        :param new_trigger: the new trigger instance
        """
        settings = SettingLoader().settings
        list_no_duplicate_triggers = [trigger for trigger in settings.triggers if trigger.name != new_trigger.name]
        list_no_duplicate_triggers.append(new_trigger)
        settings.triggers = list_no_duplicate_triggers

    # HOOKS
    @staticmethod
    def set_hooks(hooks):
        """
        Update the hooks dictionary defined in the settings with the new dictionary in param.
        :param hooks: the dictionary containing hooks to update.
        :type hooks : dict
        """
        settings = SettingLoader().settings
        settings.hooks.update(hooks)

    # Variables
    @staticmethod
    def set_variables(variables):
        """
        Update the settings variables dictionary.
        :param variables: The dict of variables with the new values.
        """
        settings = SettingLoader().settings
        settings.variables.update(variables)



# coding: utf8
import collections
from collections import Counter
import six
import yaml
from jinja2 import Template

from brain.core.NeuronParameterLoader import NeuronParameterLoader

from brain.core.Models.MatchedSynapse import MatchedSynapse
from brain.core.Utils.Utils import Utils
from brain.core.ConfigurationManager import SettingLoader

import logging

logging.basicConfig()
logger = logging.getLogger("brain")


class OrderAnalyser:
    """
    This Class is used to get a list of synapses that match a given Spoken order
    """

    brain = None
    settings = None

    @classmethod
    def __init__(cls):
        cls.settings = SettingLoader().settings

    @classmethod
    def get_matching_synapse(cls, order, brain=None):
        """
        Return the list of matching synapses from the given order
        :param order: The user order
        :param brain: The loaded brain
        :return: The List of synapses matching the given order
        """
        cls.brain = brain
        logger.debug("[OrderAnalyser] Received order: %s" % order)
        if isinstance(order, six.binary_type):
            order = order.decode('utf-8')

        # We use a namedtuple to associate the synapse and the signal of the synapse
        synapse_order_tuple = collections.namedtuple('tuple_synapse_matchingOrder',
                                                     ['synapse', 'order'])

        # if the received order is None we can stop the process immediately
        if order is None:
            return list()

        # test each synapse from the brain
        list_match_synapse = cls.get_list_match_synapse(order, synapse_order_tuple)

        # create a list of MatchedSynapse from the tuple list
        list_synapse_to_process = cls.get_list_synapses_to_process(list_match_synapse, order)

        return list_synapse_to_process

    @classmethod
    def get_list_synapses_to_process(cls, list_match_synapse, order):
        list_synapse_to_process = list()
        for tuple_el in list_match_synapse:
            new_matching_synapse = MatchedSynapse(matched_synapse=tuple_el.synapse,
                                                  matched_order=tuple_el.order,
                                                  user_order=order)
            list_synapse_to_process.append(new_matching_synapse)
        return list_synapse_to_process

    @classmethod
    def get_list_match_synapse(cls, order, synapse_order_tuple):
        list_match_synapse = list()
        for synapse in cls.brain.synapses:
            if synapse.enabled:
                for signal in synapse.signals:
                    # we are only concerned by synapse with a order type of signal
                    if signal.name == "order":
                        # get the type of matching expected, by default "normal"
                        expected_matching_type = "normal"
                        signal_order = None

                        if isinstance(signal.parameters, str) or isinstance(signal.parameters, six.text_type):
                            signal_order = signal.parameters
                        if isinstance(signal.parameters, dict):
                            try:
                                signal_order = signal.parameters["text"]
                            except KeyError:
                                logger.debug("[OrderAnalyser] Warning, missing parameter 'text' in order. "
                                             "Order will be skipped")
                                continue

                            expected_matching_type = cls.get_matching_type(signal)

                            order = cls.order_correction(order, signal)

                        if cls.is_order_matching(user_order=order,
                                                 signal_order=signal_order,
                                                 expected_order_type=expected_matching_type):
                            # the order match the synapse, we add it to the returned list
                            logger.debug("Order found! Run synapse name: %s" % synapse.name)
                            Utils.print_success("Order matched in the brain. Running synapse \"%s\"" % synapse.name)
                            list_match_synapse.append(synapse_order_tuple(synapse=synapse, order=signal_order))
        return list_match_synapse

    @classmethod
    def order_correction(cls, order, signal):

        stt_correction_list = list()

        stt_correction_file_path = cls.get_stt_correction_file_path(signal)
        stt_correction = cls.get_stt_correction(signal)

        if stt_correction_file_path is not None:
            stt_correction_list = cls.load_stt_correction_file(stt_correction_file_path)
        if stt_correction is not None:
            stt_correction_list = cls.override_stt_correction_list(stt_correction_list, stt_correction)
        if stt_correction_list:
            order = cls.override_order_with_correction(order, stt_correction_list)
        return order

    @staticmethod
    def get_stt_correction(signal):
        stt_correction = None
        try:
            stt_correction = signal.parameters["stt-correction"]
            logger.debug("[OrderAnalyser] stt-correction provided by user")
        except KeyError:
            logger.debug("[OrderAnalyser] No stt-correction provided")
        return stt_correction

    @staticmethod
    def get_stt_correction_file_path(signal):
        stt_correction_file_path = None
        try:
            stt_correction_file_path = signal.parameters["stt-correction-file"]
            logger.debug("[OrderAnalyser] stt-correction-file provided by user")
        except KeyError:
            logger.debug("[OrderAnalyser] No stt-correction-file provided")
        return stt_correction_file_path

    @staticmethod
    def get_matching_type(signal):
        expected_matching_type = "normal"
        try:
            expected_matching_type = signal.parameters["matching-type"]
        except KeyError:
            logger.debug("[OrderAnalyser] Warning, missing parameter 'matching-type' in order. "
                         "Fallback to 'normal'")
        return expected_matching_type

    @staticmethod
    def _get_split_order_without_bracket(order):
        """
        Get an order with bracket inside like: "hello my name is {{ name }}.
        return a list of string without bracket like ["hello", "my", "name", "is"]
        :param order: sentence to split
        :return: list of string without bracket
        """

        matches = Utils.find_all_matching_brackets(order)
        for match in matches:
            order = order.replace(match, "")
        # then split
        split_order = order.split()
        return split_order

    @classmethod
    def is_normal_matching(cls, user_order, signal_order):
        """
        True if :
            - all word in the user_order are present in the signal_order
        :param user_order: order from the user
        :param signal_order: order in the signal
        :return: Boolean
        """
        logger.debug("[OrderAnalyser] is_normal_matching called with user_order: %s, signal_order: %s" % (user_order,
                                                                                                          signal_order))
        split_user_order = user_order.split()
        split_signal_order_without_brackets = cls._get_split_order_without_bracket(signal_order)

        c1, c2 = Counter(split_signal_order_without_brackets), Counter(split_user_order)
        for k, n in c1.items():
            if n > c2[k]:
                return False
        return True

    @classmethod
    def is_strict_matching(cls, user_order, signal_order):
        """
        True if :
            - all word in the user_order are present in the signal_order
            - no additional word
        :param user_order: order from the user
        :param signal_order: order in the signal
        :return: Boolean
        """
        logger.debug("[OrderAnalyser] is_strict_matching called with user_order: %s, signal_order: %s" % (user_order,
                                                                                                          signal_order))
        if cls.is_normal_matching(user_order=user_order, signal_order=signal_order):

            # if the signal order contains bracket, we need to instantiate it with loaded parameters from the user order
            if Utils.is_containing_bracket(signal_order):
                signal_order = cls._get_instantiated_order_signal_from_user_order(signal_order, user_order)

            split_user_order = user_order.split()
            split_instantiated_signal = signal_order.split()

            if len(split_user_order) == len(split_instantiated_signal):
                return True

        return False

    @classmethod
    def is_ordered_strict_matching(cls, user_order, signal_order):
        """
       True if :
            - all word in the user_order are present in the signal_order
            - no additional word
            - same order as word present in signal_order
        :param user_order: order from the user
        :param signal_order: order in the signal
        :return: Boolean
        """
        logger.debug(
            "[OrderAnalyser] ordered_strict_matching called with user_order: %s, signal_order: %s" % (user_order,
                                                                                                      signal_order))
        if cls.is_normal_matching(user_order=user_order, signal_order=signal_order) and \
                cls.is_strict_matching(user_order=user_order, signal_order=signal_order):
            # if the signal order contains bracket, we need to instantiate it with loaded parameters from the user order
            if Utils.is_containing_bracket(signal_order):
                signal_order = cls._get_instantiated_order_signal_from_user_order(signal_order, user_order)

            split_user_order = user_order.split()
            split_signal_order = signal_order.split()
            return split_user_order == split_signal_order

        return False

    @classmethod
    def is_order_matching(cls, user_order, signal_order, expected_order_type="normal"):
        """
        return True if the user_order matches the signal_order following the expected_order_type
        where "expected_order_type" is in
        - normal: normal matching. all words are present in the user_order. this is the default
        - strict: only word in the user order match. no more word
        - ordered-strict: only word in the user order and in the same order
        :param user_order: order from the user
        :param signal_order: order in the signal
        :param expected_order_type: type of order (normal, strict, ordered-strict)
        :return: True if the order match
        """
        matching_type_function = {
            "normal": cls.is_normal_matching,
            "strict": cls.is_strict_matching,
            "ordered-strict": cls.is_ordered_strict_matching
        }

        # Lowercase all incoming
        user_order = user_order.lower()
        signal_order = signal_order.lower()

        if expected_order_type in matching_type_function:
            return matching_type_function[expected_order_type](user_order, signal_order)
        else:
            logger.debug("[OrderAnalyser] non existing matching-type: '%s', fallback to 'normal'" % expected_order_type)
            return matching_type_function["normal"](user_order, signal_order)

    @classmethod
    def _get_instantiated_order_signal_from_user_order(cls, signal_order, user_order):
        """
        return instantiated signal order with parameters loaded from the user order
        E.g:
        signal_order = "this is an {{ variable }}
        user_order = "this is an order"

        returned value is: "this is an order"

        :param user_order: the order from the user
        :param signal_order: the order with brackets from the synapse
        :return: jinja instantiated order from the signal
        """
        # get parameters
        parameters_from_user_order = NeuronParameterLoader.get_parameters(synapse_order=signal_order,
                                                                          user_order=user_order)
        # we load variables into the expected order from the signal
        t = Template(signal_order)
        signal_order = t.render(**parameters_from_user_order)

        return signal_order

    @classmethod
    def override_order_with_correction(cls, order, stt_correction):
        logger.debug("[OrderAnalyser] override_order_this_correction: stt_correction list: %s" % stt_correction)
        logger.debug("[OrderAnalyser] override_order_this_correction: order before correction: %s" % order)
        for correction in stt_correction:
            try:
                input_val = str(correction["input"])
                output_val = str(correction["output"])
            except KeyError as e:
                logger.debug("[OrderAnalyser] override_order_this_correction: "
                             "Missing key %s. Order will not be modified" % e)
                return order

            if str(input_val) in order.split():
                logger.debug("[OrderAnalyser] STT override '%s' by '%s'" % (input_val, output_val))
                order = order.replace(input_val, output_val)

        return order

    @classmethod
    def load_stt_correction_file(cls, stt_correction_file):
        stt_correction_file_path = Utils.get_real_file_path(stt_correction_file)
        stt_correction_file = open(stt_correction_file_path, "r")
        stt_correction = yaml.load(stt_correction_file)

        return stt_correction

    @classmethod
    def override_stt_correction_list(cls, stt_correction_list_to_override, correction_list):
        # add non existing dict
        for correction_to_check in correction_list:
            if correction_to_check["input"] not in (x["input"] for x in stt_correction_list_to_override):
                stt_correction_list_to_override.append(correction_to_check)

        # override dict with same input
        for correction_to_override in stt_correction_list_to_override:
            for correction_to_check in correction_list:
                if correction_to_check["input"] == correction_to_override["input"]:
                    correction_to_override["output"] = correction_to_check["output"]
                    break

        return stt_correction_list_to_override



import unittest

import mock

from brain.core.NeuronModule import MissingParameterException
from brain.neurons.brain.brain import Brain


class TestBrain(unittest.TestCase):

    def test_is_parameters_ok(self):
        # valid neuron with boolean
        synapse_name = "synapse_name"
        enabled = True
        with mock.patch("brain.neurons.brain.brain.Brain._update_brain"):
            brain_neuron = Brain(synapse_name=synapse_name, enabled=enabled)
            self.assertTrue(brain_neuron._is_parameters_ok())
            self.assertTrue(brain_neuron.enabled)

        # valid neuron with boolean as string
        synapse_name = "synapse_name"
        enabled = "True"
        with mock.patch("brain.neurons.brain.brain.Brain._update_brain"):
            brain_neuron = Brain(synapse_name=synapse_name, enabled=enabled)
            self.assertTrue(brain_neuron._is_parameters_ok())
            self.assertTrue(brain_neuron.enabled)

        # valid neuron with boolean as string
        synapse_name = "synapse_name"
        enabled = "true"
        with mock.patch("brain.neurons.brain.brain.Brain._update_brain"):
            brain_neuron = Brain(synapse_name=synapse_name, enabled=enabled)
            self.assertTrue(brain_neuron._is_parameters_ok())
            self.assertTrue(brain_neuron.enabled)

        # invalid neuron with no synapse name
        synapse_name = ""
        enabled = "true"
        with mock.patch("brain.neurons.brain.brain.Brain._update_brain"):
            with self.assertRaises(MissingParameterException):
                Brain(synapse_name=synapse_name, enabled=enabled)

        # invalid neuron with no synapse name
        synapse_name = "test"
        enabled = ""
        with mock.patch("brain.neurons.brain.brain.Brain._update_brain"):
            with self.assertRaises(MissingParameterException):
                Brain(synapse_name=synapse_name, enabled=enabled)

        # valid neuron but enabled bool automatically converted to False
        synapse_name = "test"
        enabled = "no a bool"
        with mock.patch("brain.neurons.brain.brain.Brain._update_brain"):
            brain_neuron = Brain(synapse_name=synapse_name, enabled=enabled)
            self.assertTrue(brain_neuron._is_parameters_ok())
            self.assertFalse(brain_neuron.enabled)


if __name__ == '__main__':
    unittest.main()



import tempfile
import unittest
import mock

from brain.core.Models import Singleton
from brain.core.Models.settings.Player import Player
from brain.core.Models.settings.Stt import Stt
from brain.core.Models.settings.Trigger import Trigger
from brain.core.Models.settings.Tts import Tts
from brain.neurons.settings import Settings


class TestSettings(unittest.TestCase):

    def setUp(self):
        self.neuron_settings = Settings()

    def tearDown(self):
        # Cleaning for settings singleton
        Singleton._instances = {}

    def test_is_parameters_ok(self):
        # TODO this code relies on the current settings.yml file, should create a full mock instead

        # tts
        self.neuron_settings.default_tts = "pico2wave"
        self.assertTrue(self.neuron_settings._is_parameters_ok())

        self.neuron_settings.text_to_speech = [{"pico2wave": {"language": "fr-FR"}}]
        self.assertTrue(self.neuron_settings._is_parameters_ok())

        # stt
        self.neuron_settings.default_stt = "google"
        self.assertTrue(self.neuron_settings._is_parameters_ok())

        self.neuron_settings.speech_to_text = [{"google": {"language": "fr-FR"}}]
        self.assertTrue(self.neuron_settings._is_parameters_ok())

        # player
        self.neuron_settings.default_player = "mplayer"
        self.assertTrue(self.neuron_settings._is_parameters_ok())

        self.neuron_settings.players = [{"mplayer": {}}]

        # trigger
        self.neuron_settings.default_trigger = "snowboy"
        self.assertTrue(self.neuron_settings._is_parameters_ok())

        # hooks
        self.neuron_settings.hooks = {"blabla": ["coucou", "test"]}
        self.assertTrue(self.neuron_settings._is_parameters_ok())
        self.neuron_settings.hooks = {"blabla": "string"}
        self.assertTrue(self.neuron_settings._is_parameters_ok())

        # variables
        tmpfile = tempfile.NamedTemporaryFile()
        self.neuron_settings.var_files = [tmpfile.name]
        self.assertTrue(self.neuron_settings._is_parameters_ok())

        # deaf
        self.neuron_settings.deaf = 60
        self.assertFalse(self.neuron_settings._is_parameters_ok())
        self.neuron_settings.deaf = "randomString"
        self.assertFalse(self.neuron_settings._is_parameters_ok())
        self.neuron_settings.deaf = 0
        self.assertFalse(self.neuron_settings._is_parameters_ok())
        self.neuron_settings.deaf = True
        self.assertTrue(self.neuron_settings._is_parameters_ok())


        # mute
        self.neuron_settings.mute = 60
        self.assertFalse(self.neuron_settings._is_parameters_ok())
        self.neuron_settings.mute = "randomString"
        self.assertFalse(self.neuron_settings._is_parameters_ok())
        self.neuron_settings.mute = 0
        self.assertFalse(self.neuron_settings._is_parameters_ok())
        self.neuron_settings.mute = True
        self.assertTrue(self.neuron_settings._is_parameters_ok())

        # threshold
        self.neuron_settings.energy_threshold = "randomString"
        self.assertFalse(self.neuron_settings._is_parameters_ok())
        self.neuron_settings.energy_threshold = 60
        self.assertTrue(self.neuron_settings._is_parameters_ok())

        # adjust_ambiant
        self.neuron_settings.adjust_for_ambient_noise_second = "randomString"
        self.assertFalse(self.neuron_settings._is_parameters_ok())
        self.neuron_settings.adjust_for_ambient_noise_second = 60
        self.assertTrue(self.neuron_settings._is_parameters_ok())

    def test_set_settings(self):
        # tts
        self.neuron_settings.default_tts = "randomtts"
        with mock.patch("brain.core.ConfigurationManager.SettingEditor.set_default_tts") as mock_setting_editor:
            self.neuron_settings._set_settings()
            mock_setting_editor.assert_called_once_with(self.neuron_settings.default_tts)

        self.neuron_settings.text_to_speech = [{"randomTTS": {"language": "fr-FR"}}]
        tts = Tts(name= "randomTTS", parameters= {"language": "fr-FR"})
        with mock.patch("brain.core.ConfigurationManager.SettingEditor.set_ttss") as mock_setting_editor:
            self.neuron_settings._set_settings()
            mock_setting_editor.assert_called_once_with(tts)

        # stt
        self.neuron_settings.default_stt = "randomstt"
        with mock.patch("brain.core.ConfigurationManager.SettingEditor.set_default_stt") as mock_setting_editor:
            self.neuron_settings._set_settings()
            mock_setting_editor.assert_called_once_with(self.neuron_settings.default_stt)

        self.neuron_settings.speech_to_text = [{"randomStt": {"language": "fr-FR"}}]
        stt = Stt(name="randomStt", parameters={"language": "fr-FR"})
        with mock.patch("brain.core.ConfigurationManager.SettingEditor.set_stts") as mock_setting_editor:
            self.neuron_settings._set_settings()
            mock_setting_editor.assert_called_once_with(stt)

        # players
        self.neuron_settings.default_player = "randomPlayer"
        with mock.patch("brain.core.ConfigurationManager.SettingEditor.set_default_player") as mock_setting_editor:
            self.neuron_settings._set_settings()
            mock_setting_editor.assert_called_once_with(self.neuron_settings.default_player)

        self.neuron_settings.players = [{"randomPlayer": {}}]
        player = Player(name="randomPlayer", parameters={})
        with mock.patch("brain.core.ConfigurationManager.SettingEditor.set_players") as mock_setting_editor:
            self.neuron_settings._set_settings()
            mock_setting_editor.assert_called_once_with(player)

        # triggers
        self.neuron_settings.default_trigger = "randomTrigger"
        with mock.patch("brain.core.ConfigurationManager.SettingEditor.set_default_trigger") as mock_setting_editor:
            self.neuron_settings._set_settings()
            mock_setting_editor.assert_called_once_with(self.neuron_settings.default_trigger)

        self.neuron_settings.triggers = [{"randomTrigger": {}}]
        trigger = Trigger(name="randomTrigger", parameters={})
        with mock.patch("brain.core.ConfigurationManager.SettingEditor.set_trigger") as mock_setting_editor:
            self.neuron_settings._set_settings()
            mock_setting_editor.assert_called_once_with(trigger)

        # Hooks
        self.neuron_settings.hooks = {"randomHook": "randomSynapse"}
        with mock.patch("brain.core.ConfigurationManager.SettingEditor.set_hooks") as mock_setting_editor:
            self.neuron_settings._set_settings()
            mock_setting_editor.assert_called_once_with(self.neuron_settings.hooks)

        # Variables
        with tempfile.NamedTemporaryFile() as tmpfile:
            tmpfile.write("coucou: 'hello'".encode()) # encode to get the binary format
            tmpfile.flush() # To refresh the file with the data
            self.neuron_settings.var_files = [tmpfile.name]
            with mock.patch("brain.core.ConfigurationManager.SettingEditor.set_variables") as mock_setting_editor:
                self.neuron_settings._set_settings()
                mock_setting_editor.assert_called_once_with({'coucou': 'hello'})
                self.neuron_settings.var_files = [] # reset var_file

        # Deaf
        self.neuron_settings.deaf = True
        with mock.patch("brain.core.ConfigurationManager.SettingEditor.set_deaf_status") as mock_setting_editor:
            with mock.patch("brain.core.SignalLauncher.SignalLauncher.get_order_instance"):
                self.neuron_settings._set_settings()
                mock_setting_editor.assert_called_once()

        # Mute
        self.neuron_settings.mute = True
        with mock.patch("brain.core.ConfigurationManager.SettingEditor.set_mute_status") as mock_setting_editor:
            self.neuron_settings._set_settings()
            mock_setting_editor.assert_called_once_with(True)

        # set_energy_threshold
        self.neuron_settings.energy_threshold = 50
        with mock.patch("brain.core.ConfigurationManager.SettingEditor.set_energy_threshold") as mock_setting_editor:
            self.neuron_settings._set_settings()
            mock_setting_editor.assert_called_once_with(50)

        # set_adjust_for_ambient_noise_second
        self.neuron_settings.adjust_for_ambient_noise_second = 3000
        with mock.patch(
                "brain.core.ConfigurationManager.SettingEditor.set_adjust_for_ambient_noise_second") as mock_setting_editor:
            self.neuron_settings._set_settings()
            mock_setting_editor.assert_called_once_with(3000)


import logging

from brain.core import NeuronModule
from brain.core.NeuronModule import MissingParameterException, InvalidParameterException
from brain.core.NotificationManager import NotificationManager

logging.basicConfig()
logger = logging.getLogger("brain")


class Signals(NeuronModule):
    def __init__(self, **kwargs):
        super(Signals, self).__init__(**kwargs)

        # get the command
        self.notification = kwargs.get('notification', False)
        self.payload = kwargs.get('payload', False)

        if self._is_parameters_ok():

            logger.debug("[Signals] Send a notification to all subscribed classes, notification: '%s', payload: %s"
                         % (self.notification, self.payload))

            NotificationManager.send_notification(self.notification, self.payload)

    def _is_parameters_ok(self):

        if self.notification is None:
            raise MissingParameterException("[Signals] This neuron require a 'notification parameter'")

        return True



import speech_recognition as sr

from brain.core import Utils
from brain.stt.Utils import SpeechRecognition


class Houndify(SpeechRecognition):

    def __init__(self, callback=None, **kwargs):
        """
        Start recording the microphone and analyse audio with Houndify api
        :param callback: The callback function to call to send the text
        :param kwargs:
        """
        # give the audio file path to process directly to the mother class if exist
        SpeechRecognition.__init__(self, kwargs.get('audio_file_path', None))

        # callback function to call after the translation speech/tex
        self.main_controller_callback = callback
        self.client_id = kwargs.get('client_id', None)
        self.key = kwargs.get('key', None)
        # only english supported
        # self.language = kwargs.get('language', "en-US")
        self.show_all = kwargs.get('show_all', False)

        # start listening in the background
        self.set_callback(self.houndify_callback)
        # start processing, record a sample from the microphone if no audio file path provided, else read the file
        self.start_processing()

    def houndify_callback(self, recognizer, audio):
        """
        called from the background thread
        """
        try:
            captured_audio = recognizer.recognize_houndify(audio,
                                                           client_id=self.client_id,
                                                           client_key=self.key,
                                                           show_all=self.show_all)
            Utils.print_success("Houndify Speech Recognition thinks you said %s" % captured_audio)
            self._analyse_audio(captured_audio)

        except sr.UnknownValueError:
            Utils.print_warning("Houndify Speech Recognition could not understand audio")
            # callback anyway, we need to listen again for a new order
            self._analyse_audio(audio_to_text=None)
        except sr.RequestError as e:
            Utils.print_danger("Could not request results from Houndify Speech Recognition service; {0}".format(e))
            # callback anyway, we need to listen again for a new order
            self._analyse_audio(audio_to_text=None)
        except AssertionError:
            Utils.print_warning("No audio caught from microphone")
            self._analyse_audio(audio_to_text=None)

    def _analyse_audio(self, audio_to_text):
        """
        Confirm the audio exists and run it in a Callback
        :param audio_to_text: the captured audio
        """
        if self.main_controller_callback is not None:
            self.main_controller_callback(audio_to_text)



from threading import Thread

import logging
import speech_recognition as sr

from brain import Utils, SettingLoader

logging.basicConfig()
logger = logging.getLogger("brain")


class SpeechRecognition(Thread):

    def __init__(self, audio_file=None):
        """
        Thread used to caught n audio from the microphone and pass it to a callback method
        """
        super(SpeechRecognition, self).__init__()
        self.recognizer = sr.Recognizer()
        self.microphone = sr.Microphone()
        self.callback = None
        self.stop_thread = None
        self.kill_yourself = False
        self.audio_stream = None

        # get global configuration
        sl = SettingLoader()
        self.settings = sl.settings

        if audio_file is None:
            # audio file not set, we need to capture a sample from the microphone
            with self.microphone as source:
                if self.settings.options.adjust_for_ambient_noise_second > 0:
                    # threshold is calculated from capturing ambient sound
                    logger.debug("[SpeechRecognition] threshold calculated by "
                                 "capturing ambient noise during %s seconds" %
                                 self.settings.options.adjust_for_ambient_noise_second)
                    Utils.print_info("[SpeechRecognition] capturing ambient sound during %s seconds" %
                                     self.settings.options.adjust_for_ambient_noise_second)
                    self.recognizer.adjust_for_ambient_noise(source,
                                                             duration=self.settings.
                                                             options.adjust_for_ambient_noise_second)
                else:
                    # threshold is defined manually
                    logger.debug("[SpeechRecognition] threshold defined by settings: %s" %
                                 self.settings.options.energy_threshold)
                    self.recognizer.energy_threshold = self.settings.options.energy_threshold

                Utils.print_info("[SpeechRecognition] Threshold set to: %s" % self.recognizer.energy_threshold)
        else:
            # audio file provided
            with sr.AudioFile(audio_file) as source:
                self.audio_stream = self.recognizer.record(source)  # read the entire audio file

    def run(self):
        """
        Start the thread that listen the microphone and then give the audio to the callback method
        """
        if self.audio_stream is None:
            Utils.print_info("Say something!")
            try:
                with self.microphone as source:
                    logger.debug("[SpeechRecognition] STT timeout: %s" % self.settings.options.stt_timeout)
                    self.audio_stream = self.recognizer.listen(source, timeout=self.settings.options.stt_timeout)
            except sr.WaitTimeoutError:
                logger.debug("[SpeechRecognition] timeout reached while waiting for audio input")
                self.audio_stream = None
            logger.debug("[SpeechRecognition] end of speech recognition process")

            self.callback(self.recognizer, self.audio_stream)

    def start_processing(self):
        """
        A method to start the thread
        """
        self.start()

    def set_callback(self, callback):
        """
        set the callback method that will receive the audio stream caught by the microphone
        :param callback: callback method
        :return:
        """
        self.callback = callback



import re
import os
import imp

from brain.core.Utils.Utils import ModuleNotFoundError
from brain.core.ConfigurationManager.SettingLoader import SettingLoader


class InvalidSynapeName(Exception):
    """
    The name of the synapse is not correct. It should only contains alphanumerics at the beginning and the end of
    its name. It can also contains dash in beetween alphanumerics.
    """
    pass


class NoSynapeName(Exception):
    """
    A synapse needs a name
    """
    pass


class NoSynapeNeurons(Exception):
    """
    A synapse must contains at least one neuron

    .. seealso:: Synapse, Neuron
    """
    pass


class NoSynapeSignals(Exception):
    """
    A synapse must contains at least an Event or an Order

    .. seealso:: Event, Order
    """
    pass


class NoValidSignal(Exception):
    """
    A synapse must contains at least a valid Event or an Order

    .. seealso:: Event, Order
    """

    pass


class MultipleSameSynapseName(Exception):
    """
    A synapse name must be unique
    """
    pass


class NoValidOrder(Exception):
    pass


class ConfigurationChecker:
    """
    This Class provides all method to Check the configuration files are properly set up.
    """

    def __init__(self):
        pass

    @staticmethod
    def check_synape_dict(synape_dict):
        """
        Return True if the provided dict is well corresponding to a Synapse

        :param synape_dict: The synapse Dictionary
        :type synape_dict: Dict
        :return: True if synapse are ok
        :rtype: Boolean

        :Example:

            ConfigurationChecker().check_synape_dict(synapses_dict):

        .. seealso:: Synapse
        .. raises:: NoSynapeName, InvalidSynapeName, NoSynapeNeurons, NoSynapeSignals
        .. warnings:: Static and Public
        """

        if 'name' not in synape_dict:
            raise NoSynapeName("The Synapse does not have a name: %s" % synape_dict)

        # check that the name is conform
        # Regex for [a - zA - Z0 - 9\-] with dashes allowed in between but not at the start or end
        pattern = r'(?=[a-zA-Z0-9\-]{4,100}$)^[a-zA-Z0-9]+(\-[a-zA-Z0-9]+)*$'
        prog = re.compile(pattern)
        result = prog.match(synape_dict["name"])
        if result is None:
            raise InvalidSynapeName("Error with synapse name \"%s\"."
                                    "Valid syntax: "
                                    "At least 4 characters "
                                    "[a - zA - Z0 - 9\-] with dashes allowed in between but not at the start or end" %
                                    synape_dict["name"])

        if 'neurons' not in synape_dict:
            raise NoSynapeNeurons("The Synapse does not have neurons: %s" % synape_dict)

        if 'signals' not in synape_dict:
            raise NoSynapeSignals("The Synapse does not have signals: %s" % synape_dict)

        return True

    @staticmethod
    def check_neuron_dict(neuron_dict):
        """
        Check received neuron dict is valid:

        :param neuron_dict: The neuron Dictionary
        :type neuron_dict: Dict
        :return: True if neuron is ok
        :rtype: Boolean

        :Example:

            ConfigurationChecker().check_neuron_dict(neurons_dict):

        .. seealso:: Synapse
        .. raises:: ModuleNotFoundError
        .. warnings:: Static and Public
        """

        def check_neuron_exist(neuron_module_name):
            """
            Return True if the neuron_name python Class exist in neurons package
            :param neuron_module_name: Name of the neuron module to check
            :type neuron_module_name: str
            :return:
            """
            sl = SettingLoader()
            settings = sl.settings
            package_name = "brain.neurons" + "." + neuron_module_name.lower() + "." + neuron_module_name.lower()
            if settings.resources.neuron_folder is not None:
                neuron_resource_path = settings.resources.neuron_folder + \
                                       os.sep + neuron_module_name.lower() + os.sep + \
                                       neuron_module_name.lower() + ".py"
                if os.path.exists(neuron_resource_path):
                    imp.load_source(neuron_module_name.capitalize(), neuron_resource_path)
                    package_name = neuron_module_name.capitalize()

            try:
                mod = __import__(package_name, fromlist=[neuron_module_name.capitalize()])
                getattr(mod, neuron_module_name.capitalize())
            except AttributeError:
                raise ModuleNotFoundError("[AttributeError] The module %s does not exist in the package %s " % (
                neuron_module_name.capitalize(),
                package_name))
            except ImportError:
                raise ModuleNotFoundError(
                    "[ImportError] The module %s does not exist in the package %s " % (neuron_module_name.capitalize(),
                                                                                       package_name))
            return True

        if isinstance(neuron_dict, dict):
            for neuron_name in neuron_dict:
                check_neuron_exist(neuron_name)
        else:
            check_neuron_exist(neuron_dict)
        return True

    @staticmethod
    def check_signal_dict(signal_dict):

        def check_signal_exist(signal_name):
            """
            Return True if the signal_name python Class exist in signals package
            :param signal_name: Name of the neuron module to check
            :type signal_name: str
            :return:
            """
            sl = SettingLoader()
            settings = sl.settings
            package_name = "brain.signals" + "." + signal_name.lower() + "." + signal_name.lower()
            if settings.resources.signal_folder is not None:
                neuron_resource_path = settings.resources.neuron_folder + \
                                       os.sep + signal_name.lower() + os.sep + \
                                       signal_name.lower() + ".py"
                if os.path.exists(neuron_resource_path):
                    imp.load_source(signal_name.capitalize(), neuron_resource_path)
                    package_name = signal_name.capitalize()

            try:
                mod = __import__(package_name, fromlist=[signal_name.capitalize()])
                getattr(mod, signal_name.capitalize())
            except AttributeError:
                raise ModuleNotFoundError(
                    "[AttributeError] The module %s does not exist in the package %s " % (signal_name.capitalize(),
                                                                                          package_name))
            except ImportError:
                raise ModuleNotFoundError(
                    "[ImportError] The module %s does not exist in the package %s " % (signal_name.capitalize(),
                                                                                       package_name))
            return True

        if isinstance(signal_dict, dict):
            for signal_name in signal_dict:
                check_signal_exist(signal_name)
        else:
            check_signal_exist(signal_dict)
        return True

    @staticmethod
    def check_order_dict(order_dict):
        """
        Check received order dictionary is valid:

        :param order_dict: The Order Dict
        :type order_dict: Dict
        :return: True if event are ok
        :rtype: Boolean

        :Example:

            ConfigurationChecker().check_order_dict(order_dict):

        .. seealso::  Order
        .. warnings:: Static and Public
        """
        if order_dict is None or order_dict == "":
            raise NoValidOrder("An order cannot be null or empty")

        return True

    @staticmethod
    def check_synapes(synapses_list):
        """
        Check the synapse list is ok:
                - No double same name

        :param synapses_list: The Synapse List
        :type synapses_list: List
        :return: list of Synapse
        :rtype: List

        :Example:

            ConfigurationChecker().check_synapes(order_dict):

        .. seealso::  Synapse
        .. raises:: MultipleSameSynapseName
        .. warnings:: Static and Public
        """

        seen = set()
        for synapse in synapses_list:
            # convert ascii to UTF-8
            synapse_name = synapse.name.encode('utf-8')
            if synapse_name in seen:
                raise MultipleSameSynapseName("Multiple synapse found with the same name: %s" % synapse_name)
            seen.add(synapse_name)

        return True



from brain.core.Models.settings.SettingsEntry import SettingsEntry


class Trigger(SettingsEntry):
    """
    This Class is representing a Trigger with its name and parameters

    .. note:: must be defined in the settings.yml
    """

    def __init__(self, name=None, parameters=None):
        super(Trigger, self).__init__(name=name)
        self.parameters = parameters

    def __str__(self):
        return str(self.serialize())

    def serialize(self):
        # TODO fix Trigger to remove "callback" from parameters
        return {
            'name': self.name,
            'parameters': dict((key, value) for key, value in self.parameters.items() if key != "callback"),
        }

    def __eq__(self, other):
        """
        This is used to compare 2 objects
        :param other: the Trigger to compare
        :return: True if both triggers are similar, False otherwise
        """
        return self.__dict__ == other.__dict__



import logging
from threading import Thread
from time import sleep

from brain.core.SignalModule import SignalModule
from brain.core.Cortex import Cortex
from brain.core.SynapseLauncher import SynapseLauncher

from brain.core.OrderListener import OrderListener

from brain import Utils, BrainLoader

from brain.core.TriggerLauncher import TriggerLauncher
from transitions import Machine

from brain.core.PlayerLauncher import PlayerLauncher

from brain.core.ConfigurationManager import SettingLoader

from brain.core.HookManager import HookManager

logging.basicConfig()
logger = logging.getLogger("brain")


class Order(SignalModule, Thread):

    states = ['init',
              'starting_trigger',
              'waiting_for_trigger_callback',
              'stopping_trigger',
              'start_order_listener',
              'waiting_for_order_listener_callback',
              'analysing_order']

    def __init__(self):
        super(SignalModule, self).__init__()
        Thread.__init__(self, name=Order)
        #Utils.print_info('Starting order signal')
        # load settings and brain from singleton
        sl = SettingLoader()
        self.settings = sl.settings
        self.brain = BrainLoader().brain

        # keep in memory the order to process
        self.order_to_process = None

        # get the player instance
        self.player_instance = PlayerLauncher.get_player(settings=self.settings)

        # save an instance of the trigger
        self.trigger_instance = None
        self.trigger_callback_called = False
        self.skip_trigger = False  # keep the status of the trigger, if true we can skip it in the statue machine

        # save the current order listener
        self.order_listener = None
        self.order_listener_callback_called = False

        # Initialize the state machine
        self.machine = Machine(model=self, states=Order.states, initial='init', queued=True)

        # define transitions
        self.machine.add_transition('start_trigger', ['init', 'analysing_order'], 'starting_trigger')
        self.machine.add_transition('wait_trigger_callback', 'starting_trigger', 'waiting_for_trigger_callback')
        self.machine.add_transition('stop_trigger', 'waiting_for_trigger_callback', 'stopping_trigger')
        self.machine.add_transition('wait_for_order', 'stopping_trigger', 'waiting_for_order_listener_callback')
        self.machine.add_transition('analyse_order', 'waiting_for_order_listener_callback', 'analysing_order')
        self.machine.add_transition('start_order_listener', 'analysing_order', 'start_order_listener')

        self.machine.add_ordered_transitions()

        # add method which are called when changing state
        self.machine.on_enter_starting_trigger('start_trigger_process')
        self.machine.on_enter_waiting_for_trigger_callback('waiting_for_trigger_callback_thread')
        self.machine.on_enter_stopping_trigger('stop_trigger_process')
        self.machine.on_enter_start_order_listener('start_order_listener_thread')
        self.machine.on_enter_waiting_for_order_listener_callback('waiting_for_order_listener_callback_thread')
        self.machine.on_enter_analysing_order('analysing_order_thread')

    def run(self):
        # run hook on_start
        HookManager.on_start()
        self.start_trigger()

    def start_trigger_process(self):
        """
        This function will start the trigger thread that listen for the hotword
        """
        logger.debug("[Order] Entering state: %s" % self.state)
        HookManager.on_waiting_for_trigger()
        self.trigger_instance = TriggerLauncher.get_trigger(settings=self.settings, callback=self.trigger_callback)
        self.trigger_callback_called = False
        self.trigger_instance.daemon = True
        # Wait that the brain trigger is pronounced by the user
        self.trigger_instance.start()
        self.next_state()

    def waiting_for_trigger_callback_thread(self):
        """
        Method to print in debug that the main process is waiting for a trigger detection
        """
        logger.debug("[Order] Entering state: %s" % self.state)
        if self.settings.options.deaf:  # the user asked to deaf inside the deaf neuron
            Utils.print_warning("Deaf mode is active")
            self.trigger_instance.pause()
        else:
            Utils.print_info("Standby for trigger detection...")
        # this loop is used to keep the main thread alive
        while not self.trigger_callback_called:
            sleep(0.1)
        # if here, then the trigger has been called
        HookManager.on_triggered()
        self.next_state()

    def waiting_for_order_listener_callback_thread(self):
        """
        Method to print in debug that the main process is waiting for an order to analyse
        """
        logger.debug("[Order] Entering state: %s" % self.state)
        # this loop is used to keep the main thread alive
        while not self.order_listener_callback_called:
            sleep(0.1)
        # TODO on end listening here
        self.next_state()

    def trigger_callback(self):
        """
        we have detected the hotword, we can now pause the Trigger for a while
        The user can speak out loud his order during this time.
        """
        logger.debug("[Order] Trigger callback called, switching to the next state")
        self.trigger_callback_called = True

    def stop_trigger_process(self):
        """
        The trigger has been awaken, we don't needed it anymore
        :return:
        """
        logger.debug("[Order] Entering state: %s" % self.state)
        self.trigger_instance.stop()
        self.next_state()

    def start_order_listener_thread(self):
        """
        Start the STT engine thread
        """
        logger.debug("[Order] Entering state: %s" % self.state)
        HookManager.on_start_listening()
        # start listening for an order
        self.order_listener_callback_called = False
        self.order_listener = OrderListener(callback=self.order_listener_callback)
        self.order_listener.daemon = True
        self.order_listener.start()
        self.next_state()

    def order_listener_callback(self, order):
        """
        Receive an order, try to retrieve it in the brain.yml to launch to attached plugins
        :param order: the sentence received
        :type order: str
        """
        logger.debug("[Order] Order listener callback called. Order to process: %s" % order)
        HookManager.on_stop_listening()
        self.order_to_process = order
        self.order_listener_callback_called = True
        # save in brain memory the last order
        Cortex.save('kalliope_last_order', order)

    def analysing_order_thread(self):
        """
        Start the order analyser with the caught order to process
        """
        if self.order_to_process is None or self.order_to_process == "":
            logger.debug("[Order] No audio caught from analysing_order_thread")
            HookManager.on_stt_error()
        else:
            logger.debug("[Order] order in analysing_order_thread %s" % self.order_to_process)
            SynapseLauncher.run_matching_synapse_from_order(self.order_to_process,
                                                            self.brain,
                                                            self.settings,
                                                            is_api_call=False)

        if self.skip_trigger:
            self.start_order_listener()
        else:
            self.start_trigger()

    def on_notification_received(self, notification=None, payload=None):
        logger.debug("[Order] received notification, notification: %s, payload: %s" % (notification, payload))
        if notification == "skip_trigger":
            if "status" in payload:
                if payload["status"] == "True":
                    logger.debug("[Order] switch signals to True")
                    self.skip_trigger = True
                if payload["status"] == "False":
                    logger.debug("[Order] switch signals to False")
                    self.skip_trigger = False



import logging
import weakref

logging.basicConfig()
logger = logging.getLogger("brain")


class NotificationManager(object):
    """
    Class sued to send messages to all instantiated object that use it as parent class
    """
    _instances = set()

    def __init__(self):
        self._instances.add(weakref.ref(self))
        logger.debug("[NotificationManager] Add new instance to the manager")

    @classmethod
    def get_instances(cls):
        dead = set()
        for ref in cls._instances:
            obj = ref()
            if obj is not None:
                yield obj
            else:
                dead.add(ref)
        cls._instances -= dead

    @classmethod
    def send_notification(cls, notification=None, payload=None):
        logger.debug("[NotificationManager] send notification to all child: notification: %s, payload: %s"
                     % (notification, payload))
        for instance in cls.get_instances():
            try:
                instance.on_notification_received(notification=notification, payload=payload)
            except NotImplementedError:
                logger.debug("[NotificationManager] The signal %s does not implement send_notification method"
                             % instance.__class__.__name__)
                pass



import logging

from brain import SignalLauncher, Utils
from brain.core import NeuronModule
from brain.core.ConfigurationManager import SettingEditor, YAMLLoader
from brain.core.Models.settings.Stt import Stt
from brain.core.Models.settings.Trigger import Trigger
from brain.core.Models.settings.Tts import Tts
from brain.core.Models.settings.Player import Player

logging.basicConfig()
logger = logging.getLogger("brain")


class Settings(NeuronModule):
    """
    This Neuron allows to update dynamically the settings of brain.
    Currently available Settings parameters :
        - default_tts
        - default_stt
        - default_trigger
        - default_player
        - text_to_speech
        - speech_to_text
        - triggers
        - players
        - hooks
        - var_files
        - variable
        - deaf
        - mute
        - energy_threshold
        - adjust_for_ambient_noise_second
    """

    def __init__(self, **kwargs):
        super(Settings, self).__init__(**kwargs)

        # Modules
        self.default_tts = kwargs.get("default_tts", None)
        self.text_to_speech = kwargs.get("text_to_speech", None)

        self.default_stt = kwargs.get("default_stt", None)
        self.speech_to_text = kwargs.get("speech_to_text", None)

        self.default_trigger = kwargs.get("default_trigger", None)
        self.triggers = kwargs.get("triggers", None)

        self.default_player = kwargs.get("default_player", None)
        self.players = kwargs.get("players", None)

        # Options
        self.deaf = kwargs.get("deaf", None)
        self.mute = kwargs.get("mute", None)
        self.energy_threshold = kwargs.get("energy_threshold", None)
        self.adjust_for_ambient_noise_second = kwargs.get("adjust_for_ambient_noise_second", None)

        # Hooks
        self.hooks = kwargs.get("hooks", None)

        # Variables
        self.var_files = kwargs.get("var_files", None)
        self.variable = kwargs.get("variable", None)

        # Not applicable yet as Variables are applied during brainloading.
        # REST API
        # RESSOURCES Does it even make sense to update this one ?

        if self._is_parameters_ok():
            self._set_settings()

    def _is_parameters_ok(self):
        """
        Check the validity for each parameter
        :return: True if all parameters are set correctly, False otherwise.
        """

        # Players
        if self.default_player:
            if not SettingEditor._check_name_in_list_settings_entry(self.default_player, self.settings.players):
                logger.debug("[Settings] default_player %s is not defined in settings file ",
                             self.default_player)
                return False

        if self.players:
            if not isinstance(self.players, list):
                logger.debug("[Settings] players current type: %s. players should be a list", type(self.players))
                return False
            for player_el in self.players:
                if not isinstance(player_el, dict):
                    logger.debug("[Settings] player current element type: %s. player element should be a dict",
                                 type(player_el))
                    return False

        # STT
        if self.default_stt:
            if not SettingEditor._check_name_in_list_settings_entry(self.default_stt, self.settings.stts):
                logger.debug("[Settings] default_stt %s is not defined in settings file ", self.default_stt)
                return False

        if self.speech_to_text:
            if not isinstance(self.speech_to_text, list):
                logger.debug("[Settings] speech_to_text current type: %s. speech_to_text should be a list",
                             type(self.speech_to_text))
                return False
            for stt_el in self.speech_to_text:
                if not isinstance(stt_el, dict):
                    logger.debug(
                        "[Settings] speech_to_text current element type: %s. speech_to_text element should be a dict",
                        type(stt_el))
                    return False

        # TRIGGER
        if self.default_trigger:
            if not SettingEditor._check_name_in_list_settings_entry(self.default_trigger, self.settings.triggers):
                logger.debug("[Settings] default_trigger %s is not defined in settings file ",
                             self.default_trigger)
                return False

        if self.triggers:
            if not isinstance(self.triggers, list):
                logger.debug("[Settings] triggers current type: %s. triggers should be a list", type(self.triggers))
                return False
            for trigger_el in self.triggers:
                if not isinstance(trigger_el, dict):
                    logger.debug("[Settings] triggers current element type: %s. triggers element should be a dict",
                                 type(trigger_el))
                    return False

        # TTS
        if self.default_tts:
            if not SettingEditor._check_name_in_list_settings_entry(self.default_tts, self.settings.ttss):
                logger.debug("[Settings] default_tts %s is not defined in settings file ", self.default_tts)
                return False

        if self.text_to_speech:
            if not isinstance(self.text_to_speech, list):
                logger.debug("[Settings] text_to_speech current type: %s. text_to_speech should be a list",
                             type(self.text_to_speech))
                return False
            for tts_el in self.text_to_speech:
                if not isinstance(tts_el, dict):
                    logger.debug(
                        "[Settings] text_to_speech element current type: %s. text_to_speech element should be a dict",
                        type(tts_el))
                    return False

        # Options
        if self.deaf is not None:
            if not isinstance(self.deaf, bool):
                logger.debug("[Settings] deaf %s is not a correct value, you must define True or False", self.deaf)
                return False

        if self.mute is not None:
            if not isinstance(self.mute, bool):
                logger.debug("[Settings] mute %s is not a correct value, you must define True or False", self.mute)
                return False

        if self.energy_threshold is not None:
            if not isinstance(self.energy_threshold, int):
                logger.debug("[Settings] energy_threshold %s is not a correct integer, you must define a number",
                             self.energy_threshold)
                return False

        if self.adjust_for_ambient_noise_second is not None:
            if not isinstance(self.adjust_for_ambient_noise_second, int):
                logger.debug(
                    "[Settings] adjust_for_ambient_noise_second %s is not a correct integer, you must define a number",
                    self.adjust_for_ambient_noise_second)
                return False

        # Hooks
        if self.hooks:
            if not isinstance(self.hooks, dict):
                logger.debug("[Settings] hooks property %s is not a dictionary as it should be.", type(self.hooks))
                return False
            for hook_name, synap in self.hooks.items():
                if not isinstance(synap, str) and not isinstance(synap, list):
                    logger.debug(
                        "[Settings] for hook element %s the type %s is nor a string nor a list as it should be.",
                        hook_name, type(synap))
                    return False

        # Variables
        if self.var_files:
            if not isinstance(self.var_files, list):
                logger.debug("[Settings] var_files property %s is not a list as it should be.", type(self.var_files))
                return False
            for file_name in self.var_files:
                var = Utils.get_real_file_path(file_name)
                if var is None:
                    logger.debug("[Settings] Variables file %s not found", file_name)
                    return False

        if self.variable:
            if not isinstance(self.variable, dict):
                logger.debug("[Settings] variable property %s is not a dict as it should be.", type(self.variable))
                return False

        return True

    def _set_settings(self):
        # PLAYERS
        if self.default_player:
            SettingEditor.set_default_player(self.default_player)

        if self.players:
            for player_el in self.players:
                if isinstance(player_el, dict):
                    for player_name in player_el:
                        name = player_name
                        parameters = player_el[name]
                        new_player = Player(name=name, parameters=parameters)
                        SettingEditor.set_players(new_player)

        # STT
        if self.default_stt:
            SettingEditor.set_default_stt(self.default_stt)

        if self.speech_to_text:
            for stt_el in self.speech_to_text:
                if isinstance(stt_el, dict):
                    for stt_name in stt_el:
                        name = stt_name
                        parameters = stt_el[name]
                        new_stt = Stt(name=name, parameters=parameters)
                        SettingEditor.set_stts(new_stt)

        # TRIGGER
        if self.default_trigger:
            SettingEditor.set_default_trigger(self.default_trigger)

        if self.triggers:
            for trigger_el in self.triggers:
                if isinstance(trigger_el, dict):
                    for trigger_name in trigger_el:
                        name = trigger_name
                        parameters = trigger_el[name]
                        new_trigger = Trigger(name=name, parameters=parameters)
                        SettingEditor.set_trigger(new_trigger)

        # TTS
        if self.default_tts:
            SettingEditor.set_default_tts(self.default_tts)

        if self.text_to_speech:
            for tts_el in self.text_to_speech:
                if isinstance(tts_el, dict):
                    for tts_name in tts_el:
                        name = tts_name
                        parameters = tts_el[name]
                        new_tts = Tts(name=name, parameters=parameters)
                        SettingEditor.set_ttss(new_tts)

        # Options
        if self.deaf is not None:
            signal_order = SignalLauncher.get_order_instance()
            if signal_order is not None:
                SettingEditor.set_deaf_status(signal_order.trigger_instance, self.deaf)

        if self.mute is not None:
            SettingEditor.set_mute_status(self.mute)

        if self.energy_threshold is not None:
            SettingEditor.set_energy_threshold(self.energy_threshold)

        if self.adjust_for_ambient_noise_second is not None:
            SettingEditor.set_adjust_for_ambient_noise_second(self.adjust_for_ambient_noise_second)

        # Hooks
        if self.hooks:
            SettingEditor.set_hooks(self.hooks)

        # Variables
        if self.var_files:
            variables = dict()
            for files in self.var_files:
                var = Utils.get_real_file_path(files)
                # var is None has been checked previously in _is_parameters_ok() method
                variables.update(YAMLLoader.get_config(var))
            SettingEditor.set_variables(variables)

        if self.variable is not None:
            SettingEditor.set_variables(self.variable)


from brain.core.OrderAnalyser import OrderAnalyser
from brain.core.OrderListener import OrderListener
from brain.core.Utils.Utils import Utils
from brain.core.Utils import FileManager
from brain.core.ResourcesManager import ResourcesManager
from brain.core.NeuronLauncher import NeuronLauncher
from brain.core.SynapseLauncher import SynapseLauncher
from brain.core.Lifo.LIFOBuffer import LIFOBuffer
from brain.core.NeuronParameterLoader import NeuronParameterLoader
from brain.core.NeuronModule import NeuronModule
from brain.core.SignalModule import SignalModule, MissingParameter
from brain.core.PlayerModule import PlayerModule
from brain.core.HookManager import HookManager
from brain.core.Lifo.LIFOBuffer import LIFOBuffer
from brain.core.Lifo.LifoManager import LifoManager


from brain.core.Models.settings.SettingsEntry import SettingsEntry


class Options(SettingsEntry):
    """
    This Class is representing an Option element with parameters and values

    .. note:: must be defined in the settings.yml
    """

    def __init__(self, energy_threshold=4000, adjust_for_ambient_noise_second=0, deaf=None, mute=None, stt_timeout=0):
        super(Options, self).__init__(name="Options")
        self.deaf = deaf
        self.mute = mute
        self.energy_threshold = energy_threshold
        self.adjust_for_ambient_noise_second = adjust_for_ambient_noise_second
        self.stt_timeout = stt_timeout

    def __str__(self):
        return str(self.serialize())

    def serialize(self):
        return {
            'name': self.name,
            'energy_threshold': self.energy_threshold,
            'adjust_for_ambient_noise_second': self.adjust_for_ambient_noise_second,
            'deaf': self.deaf,
            'mute': self.mute,
            'stt_timeout': self.stt_timeout
        }

    def __eq__(self, other):
        """
        This is used to compare 2 objects
        :param other: the Options to compare
        :return: True if both Options are similar, False otherwise
        """
        return self.__dict__ == other.__dict__



import logging

from brain import Utils, BrainLoader
from brain.core import NeuronModule
from brain.core.NeuronModule import MissingParameterException

logging.basicConfig()
logger = logging.getLogger("brain")


class Brain(NeuronModule):

    def __init__(self, **kwargs):
        super(Brain, self).__init__(**kwargs)

        self.synapse_name = kwargs.get('synapse_name', None)
        self.enabled = kwargs.get('enabled', None)

        if self._is_parameters_ok():
            self.say(self._update_brain())

    def _is_parameters_ok(self):
        """
        Check if received parameters are ok to perform operations in the neuron
        :return: true if parameters are ok, raise an exception otherwise

        .. raises:: MissingParameterException
        """
        if self.synapse_name is None or self.synapse_name == "":
            raise MissingParameterException("[Brain neuron] You must specify a 'synapse_name'")
        if self.enabled is None or self.enabled == "":
            raise MissingParameterException("[Brain neuron] You must specify a 'enabled' boolean")

        self.enabled = Utils.str_to_bool(self.enabled)

        return True

    def _update_brain(self):
        new_status = "unknown"
        brain = BrainLoader().brain
        if self.enabled:
            if brain.enable_synapse_by_name(self.synapse_name):
                new_status = "enabled"
        else:
            if brain.disable_synapse_by_name(self.synapse_name):
                new_status = "disabled"

        message = {
            "synapse_name": self.synapse_name,
            "status": new_status
        }
        return message



import logging
import os
import threading
import time

from flask import jsonify
from flask import request
from flask_cors import CORS
from werkzeug.utils import secure_filename

from brain import Utils
from brain.core.SignalLauncher import SignalLauncher
from brain._version import version_str
from brain.core.ConfigurationManager import SettingLoader, BrainLoader, SettingEditor
from brain.core.Lifo.LifoManager import LifoManager
from brain.core.Models.MatchedSynapse import MatchedSynapse
from brain.core.OrderListener import OrderListener
from brain.core.RestAPI.utils import requires_auth
from brain.core.SynapseLauncher import SynapseLauncher
from brain.core.Utils.FileManager import FileManager

logging.basicConfig()
logger = logging.getLogger("brain")

UPLOAD_FOLDER = '/tmp/brain/tmp_uploaded_audio'
ALLOWED_EXTENSIONS = {'mp3', 'wav'}


class FlaskAPI(threading.Thread):
    def __init__(self, app, port=5000, brain=None, allowed_cors_origin=False):
        """

        :param app: Flask API
        :param port: Port to listen
        :param brain: Brain object
        :type brain: Brain
        """
        super(FlaskAPI, self).__init__()
        self.app = app
        self.port = port
        self.brain = brain
        self.allowed_cors_origin = allowed_cors_origin

        # get current settings
        sl = SettingLoader()
        self.settings = sl.settings

        # api_response sent by the Order Analyser when using the /synapses/start/audio URL
        self.api_response = None
        # boolean used to notify the main process that we get the list of returned synapse
        self.order_analyser_return = False

        # configure the upload folder
        app.config['UPLOAD_FOLDER'] = UPLOAD_FOLDER
        # create the temp folder
        FileManager.create_directory(UPLOAD_FOLDER)

        # Flask configuration remove default Flask behaviour to encode to ASCII
        self.app.url_map.strict_slashes = False
        self.app.config['JSON_AS_ASCII'] = False

        if self.allowed_cors_origin is not False:
            CORS(app, resources={r"/*": {"origins": allowed_cors_origin}}, supports_credentials=True)

        # Add routing rules
        self.app.add_url_rule('/', view_func=self.get_main_page, methods=['GET'])

        # Synapses
        self.app.add_url_rule('/synapses', view_func=self.get_synapses, methods=['GET'])
        self.app.add_url_rule('/synapses/<synapse_name>', view_func=self.get_synapse, methods=['GET'])
        self.app.add_url_rule('/synapses/start/id/<synapse_name>', view_func=self.run_synapse_by_name, methods=['POST'])
        self.app.add_url_rule('/synapses/start/order', view_func=self.run_synapse_by_order, methods=['POST'])
        self.app.add_url_rule('/synapses/start/audio', view_func=self.run_synapse_by_audio, methods=['POST'])

        # Life Cycle
        self.app.add_url_rule('/shutdown/', view_func=self.shutdown_server, methods=['POST'])

        # Settings
        self.app.add_url_rule('/settings', view_func=self.get_current_settings, methods=['GET'])
        self.app.add_url_rule('/settings/deaf/', view_func=self.get_deaf, methods=['GET'])
        self.app.add_url_rule('/settings/deaf/', view_func=self.set_deaf, methods=['POST'])
        self.app.add_url_rule('/settings/mute/', view_func=self.get_mute, methods=['GET'])
        self.app.add_url_rule('/settings/mute/', view_func=self.set_mute, methods=['POST'])
        self.app.add_url_rule('/settings/ambient_noise_second/', view_func=self.get_ambient_noise_second,
                              methods=['GET'])
        self.app.add_url_rule('/settings/ambient_noise_second/', view_func=self.set_adjust_for_ambient_noise_second,
                              methods=['POST'])
        self.app.add_url_rule('/settings/energy_threshold/', view_func=self.get_energy_threshold,
                              methods=['GET'])
        self.app.add_url_rule('/settings/energy_threshold/', view_func=self.set_energy_threshold,
                              methods=['POST'])
        self.app.add_url_rule('/settings/default_tts/', view_func=self.get_default_tts,
                              methods=['GET'])
        self.app.add_url_rule('/settings/default_tts/', view_func=self.set_default_tts,
                              methods=['POST'])
        self.app.add_url_rule('/settings/default_stt/', view_func=self.get_default_stt,
                              methods=['GET'])
        self.app.add_url_rule('/settings/default_stt/', view_func=self.set_default_stt,
                              methods=['POST'])
        self.app.add_url_rule('/settings/default_player/', view_func=self.get_default_player,
                              methods=['GET'])
        self.app.add_url_rule('/settings/default_player/', view_func=self.set_default_player,
                              methods=['POST'])
        self.app.add_url_rule('/settings/default_trigger/', view_func=self.get_default_trigger,
                              methods=['GET'])
        self.app.add_url_rule('/settings/default_trigger/', view_func=self.set_default_trigger,
                              methods=['POST'])
        self.app.add_url_rule('/settings/hooks/', view_func=self.get_hooks,
                              methods=['GET'])
        self.app.add_url_rule('/settings/hooks/', view_func=self.set_hooks,
                              methods=['POST'])
        self.app.add_url_rule('/settings/variables/', view_func=self.get_variables,
                              methods=['GET'])
        self.app.add_url_rule('/settings/variables/', view_func=self.set_variables,
                              methods=['POST'])

    def run(self):
        self.app.run(host='0.0.0.0', port=int(self.port), debug=True, threaded=True, use_reloader=False)

    @requires_auth
    def get_main_page(self):
        logger.debug("[FlaskAPI] get_main_page")
        data = {
            "Brain.ai version": "%s" % version_str
        }
        return jsonify(data), 200

    @staticmethod
    def allowed_file(filename):
        return '.' in filename and \
               filename.rsplit('.', 1)[1].lower() in ALLOWED_EXTENSIONS

    def _get_synapse_by_name(self, synapse_name):
        """
        Find a synapse in the brain by its name
        :param synapse_name:
        :return:
        """
        all_synapse = self.brain.synapses
        for synapse in all_synapse:
            try:
                if synapse.name == synapse_name:
                    return synapse
            except KeyError:
                pass
        return None

    @requires_auth
    def get_synapses(self):
        """
        get all synapses.
        test with curl:
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/synapses
        """
        logger.debug("[FlaskAPI] get_synapses: all")
        data = jsonify(synapses=[e.serialize() for e in self.brain.synapses])
        return data, 200

    @requires_auth
    def get_synapse(self, synapse_name):
        """
        get a synapse by its name
        test with curl:
        curl --user admin:secret -i -X GET  http://127.0.0.1:5000/synapses/say-hello-en
        """
        logger.debug("[FlaskAPI] get_synapse: synapse_name -> %s" % synapse_name)
        synapse_target = self._get_synapse_by_name(synapse_name)
        if synapse_target is not None:
            data = jsonify(synapses=synapse_target.serialize())
            return data, 200

        data = {
            "synapse name not found": "%s" % synapse_name
        }
        return jsonify(error=data), 404

    @requires_auth
    def run_synapse_by_name(self, synapse_name):
        """
        Run a synapse by its name
        test with curl:
        curl -i --user admin:secret -X POST  http://127.0.0.1:5000/synapses/start/id/say-hello-fr

        run a synapse without making brain speaking
        curl -i -H "Content-Type: application/json" --user admin:secret -X POST  \
        -d '{"mute":"true"}' http://127.0.0.1:5000/synapses/start/id/say-hello-fr

        Run a synapse by its name and pass order's parameters
        curl -i -H "Content-Type: application/json" --user admin:secret -X POST  \
        -d '{"mute":"true", "parameters": {"parameter1": "value1" }}' \
        http://127.0.0.1:5000/synapses/start/id/say-hello-fr

        :param synapse_name: name(id) of the synapse to execute
        :return:
        """
        # get a synapse object from the name
        logger.debug("[FlaskAPI] run_synapse_by_name: synapse name -> %s" % synapse_name)
        synapse_target = BrainLoader().brain.get_synapse_by_name(synapse_name=synapse_name)

        # Store the mute value, then apply depending of the request parameters
        old_mute_value = self.settings.options.mute
        mute = self.get_value_flag_from_request(http_request=request,
                                                flag_to_find="mute",
                                                is_boolean=True)
        if mute is not None:
            SettingEditor.set_mute_status(mute=mute)

        # get parameters
        parameters = self.get_parameters_from_request(request)

        if synapse_target is None:
            data = {
                "synapse name not found": "%s" % synapse_name
            }
            if mute is not None:
                SettingEditor.set_mute_status(mute=old_mute_value)
            return jsonify(error=data), 404
        else:
            # generate a MatchedSynapse from the synapse
            matched_synapse = MatchedSynapse(matched_synapse=synapse_target, overriding_parameter=parameters)
            # get the current LIFO buffer from the singleton
            lifo_buffer = LifoManager.get_singleton_lifo()
            lifo_buffer.add_synapse_list_to_lifo([matched_synapse])
            response = lifo_buffer.execute(is_api_call=True)
            data = jsonify(response)
            if mute is not None:
                SettingEditor.set_mute_status(mute=old_mute_value)
            return data, 201

    @requires_auth
    def run_synapse_by_order(self):
        """
        Give an order to Brain.ai via API like it was from a spoken one
        Test with curl
        curl -i --user admin:secret -H "Content-Type: application/json" -X POST \
        -d '{"order":"my order"}' http://localhost:5000/synapses/start/order

        In case of quotes in the order or accents, use a file
        cat post.json:
        {"order":"j'aime"}
        curl -i --user admin:secret -H "Content-Type: application/json" -X POST \
        --data @post.json http://localhost:5000/order/

        Can be used with mute flag
        curl -i --user admin:secret -H "Content-Type: application/json" -X POST \
        -d '{"order":"my order", "mute":"true"}' http://localhost:5000/synapses/start/order

        :return:
        """
        if not request.get_json() or 'order' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'order' not set"
            }
            return jsonify(error=data), 400

        order = request.get_json('order')

        # Store the mute value, then apply depending of the request parameters
        old_mute_value = self.settings.options.mute
        mute = self.get_value_flag_from_request(http_request=request,
                                                flag_to_find="mute",
                                                is_boolean=True)
        if mute is not None:
            SettingEditor.set_mute_status(mute=mute)

        if order is not None:
            # get the order
            order_to_run = order["order"]
            logger.debug("[FlaskAPI] run_synapse_by_order: order to run -> %s" % order_to_run)
            api_response = SynapseLauncher.run_matching_synapse_from_order(order_to_run,
                                                                           self.brain,
                                                                           self.settings,
                                                                           is_api_call=True)

            data = jsonify(api_response)
            if mute is not None:
                SettingEditor.set_mute_status(mute=old_mute_value)
            return data, 201
        else:
            data = {
                "error": "order cannot be null"
            }
            if mute is not None:
                SettingEditor.set_mute_status(mute=old_mute_value)
            return jsonify(error=data), 400

    @requires_auth
    def run_synapse_by_audio(self):
        """
        Give an order to Brain.ai with an audio file
        Test with curl
        curl -i --user admin:secret -X POST  http://localhost:5000/synapses/start/audio -F "file=@/path/to/input.wav"

        With mute flag
        curl -i --user admin:secret -X POST http://localhost:5000/synapses/start/audio -F "file=@path/to/file.wav" -F mute="true"
        :return:
        """

        # check if the post request has the file part
        if 'file' not in request.files:
            data = {
                "error": "No file provided"
            }
            return jsonify(error=data), 400

        uploaded_file = request.files['file']
        # if user does not select file, browser also
        # submit a empty part without filename
        if uploaded_file.filename == '':
            data = {
                "error": "No file provided"
            }
            return jsonify(error=data), 400

        # Store the mute value, then apply depending of the request parameters
        old_mute_value = self.settings.options.mute
        if request.form.get("mute"):
            SettingEditor.set_mute_status(mute=Utils.str_to_bool(request.form.get("mute")))

        # save the file
        filename = secure_filename(uploaded_file.filename)
        base_path = os.path.join(self.app.config['UPLOAD_FOLDER'])
        uploaded_file.save(os.path.join(base_path, filename))

        # now start analyse the audio with STT engine
        audio_path = base_path + os.sep + filename
        logger.debug("[FlaskAPI] run_synapse_by_audio: with file path %s" % audio_path)
        if not self.allowed_file(audio_path):
            audio_path = self._convert_to_wav(audio_file_path=audio_path)
        ol = OrderListener(callback=self.audio_analyser_callback, audio_file_path=audio_path)
        ol.start()
        ol.join()
        # wait the Order Analyser processing. We need to wait in this thread to keep the context
        while not self.order_analyser_return:
            time.sleep(0.1)
        self.order_analyser_return = False
        if self.api_response is not None and self.api_response:
            data = jsonify(self.api_response)
            self.api_response = None
            logger.debug("[FlaskAPI] run_synapse_by_audio: data %s" % data)
            if request.form.get("mute"):
                SettingEditor.set_mute_status(mute=old_mute_value)
            return data, 201
        else:
            data = {
                "error": "The given order doesn't match any synapses"
            }
            if request.form.get("mute"):
                SettingEditor.set_mute_status(mute=old_mute_value)
            return jsonify(error=data), 400

    @staticmethod
    def _convert_to_wav(audio_file_path):
        """
        If not already .wav, convert an incoming audio file to wav format. Using system avconv (raspberry)
        :param audio_file_path: the current full file path
        :return: Wave file path
        """
        # Not allowed so convert into wav using avconv (raspberry)
        base = os.path.splitext(audio_file_path)[0]
        extension = os.path.splitext(audio_file_path)[1]
        if extension != ".wav":
            current_file_path = audio_file_path
            audio_file_path = base + ".wav"
            os.system("avconv -y -i " + current_file_path + " " + audio_file_path)  # --> deprecated
            # subprocess.call(['avconv', '-y', '-i', audio_path, new_file_path], shell=True) # Not working ...

        return audio_file_path

    @requires_auth
    def shutdown_server(self):
        func = request.environ.get('werkzeug.server.shutdown')
        if func is None:
            raise RuntimeError('Not running with the Werkzeug Server')
        func()
        return "Shutting down..."

    @requires_auth
    def get_deaf(self):
        """
        Return the current trigger status

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/deaf
        """

        # find the order signal and call the deaf settings
        if self.settings.options.deaf is not None:
            data = {
                "deaf": self.settings.options.deaf
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "deaf status unknow"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_deaf(self):
        """
        Set the trigger status (deaf or not)

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"deaf": "True"}' http://127.0.0.1:5000/deaf
        """

        if not request.get_json() or 'deaf' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'deaf' not set"
            }
            return jsonify(error=data), 400

        # get deaf if present
        deaf = self.get_value_flag_from_request(http_request=request,
                                                flag_to_find="deaf",
                                                is_boolean=True)

        signal_order = SignalLauncher.get_order_instance()
        if signal_order is not None and deaf is not None and self.settings.options.deaf is not None:
            SettingEditor.set_deaf_status(signal_order.trigger_instance, deaf)
            data = {
                "deaf": self.settings.options.deaf
            }
            return jsonify(data), 200

        data = {
            "error": "Cannot switch deaf status"
        }
        return jsonify(error=data), 400

    @requires_auth
    def get_mute(self):
        """
        Return the current mute status

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/mute
        """

        # find the order signal and call the mute settings
        if self.settings.options.mute is not None:
            data = {
                "mute": self.settings.options.mute
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "mute status unknow"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_mute(self):
        """
        Set the Brain.ai Core mute status (mute or not)

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"mute": "True"}' http://127.0.0.1:5000/mute
        """

        if not request.get_json() or 'mute' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'mute' not set"
            }
            return jsonify(error=data), 400

        # get mute if present
        mute = self.get_value_flag_from_request(http_request=request,
                                                flag_to_find="mute",
                                                is_boolean=True)
        SettingEditor.set_mute_status(mute=mute)

        data = {
            "mute": mute
        }
        return jsonify(data), 200

    @requires_auth
    def get_energy_threshold(self):
        """
        Return the current energy_threshold value from settings

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings/energy_threshold
        """

        if self.settings.options.energy_threshold is not None:
            data = {
                "energy_threshold": self.settings.options.energy_threshold
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "energy_threshold status not defined"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_energy_threshold(self):
        """
        Set the Brain.ai Core energy_threshold value

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"energy_threshold": "6666"}' http://127.0.0.1:5000/settings/energy_threshold
        """

        if not request.get_json() or 'energy_threshold' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'energy_threshold' not set"
            }
            return jsonify(error=data), 400

        # get energy_threshold if present
        energy_threshold = self.get_value_flag_from_request(http_request=request,
                                                            flag_to_find="energy_threshold",
                                                            is_boolean=False)

        SettingEditor.set_energy_threshold(energy_threshold=energy_threshold)

        data = {
            "energy_threshold": energy_threshold
        }
        return jsonify(data), 200

    @requires_auth
    def get_ambient_noise_second(self):
        """
        Return the current ambient_noise_second value from settings

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings/ambient_noise_second
        """

        if self.settings.options.adjust_for_ambient_noise_second is not None:
            data = {
                "ambient_noise_second": self.settings.options.adjust_for_ambient_noise_second
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "ambient_noise_second status not defined"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_adjust_for_ambient_noise_second(self):
        """
        Set the Brain.ai Core ambient_noise_second value

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"energy_threshold": "6666"}' http://127.0.0.1:5000/settings/ambient_noise_second
        """

        if not request.get_json() or 'ambient_noise_second' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'ambient_noise_second' not set"
            }
            return jsonify(error=data), 400

        # get if present
        ambient_noise_second = self.get_value_flag_from_request(http_request=request,
                                                                flag_to_find="ambient_noise_second",
                                                                is_boolean=False)

        SettingEditor.set_adjust_for_ambient_noise_second(adjust_for_ambient_noise_second=ambient_noise_second)

        data = {
            "ambient_noise_second": ambient_noise_second
        }
        return jsonify(data), 200

    @requires_auth
    def get_default_tts(self):
        """
        Return the current default_tts value from settings

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings/default_tts
        """

        # the default_tts settings
        if self.settings.default_tts_name is not None:
            data = {
                "default_tts": self.settings.default_tts_name
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "default_tts status not defined"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_default_tts(self):
        """
        Set the Brain.ai Core default_tts value

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"default_tts": "myTTS"}' http://127.0.0.1:5000/settings/default_tts
        """

        if not request.get_json() or 'default_tts' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'default_tts' not set"
            }
            return jsonify(error=data), 400

        # get if present
        value = self.get_value_flag_from_request(http_request=request,
                                                 flag_to_find="default_tts",
                                                 is_boolean=False)

        SettingEditor.set_default_tts(default_tts_name=value)

        data = {
            "default_tts": self.settings.default_tts_name
        }
        return jsonify(data), 200

    @requires_auth
    def get_default_stt(self):
        """
        Return the current default_stt value from settings

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings/default_stt
        """

        # the default_tts settings
        if self.settings.default_stt_name is not None:
            data = {
                "default_stt": self.settings.default_stt_name
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "default_stt status not defined"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_default_stt(self):
        """
        Set the Brain.ai Core default_stt value

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"default_stt": "myStt"}' http://127.0.0.1:5000/settings/default_stt
        """

        if not request.get_json() or 'default_stt' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'default_stt' not set"
            }
            return jsonify(error=data), 400

        # get if present
        value = self.get_value_flag_from_request(http_request=request,
                                                 flag_to_find="default_stt",
                                                 is_boolean=False)

        SettingEditor.set_default_stt(default_stt_name=value)

        data = {
            "default_stt": self.settings.default_stt_name
        }
        return jsonify(data), 200

    @requires_auth
    def get_default_player(self):
        """
        Return the current default_player value from settings

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings/default_player
        """

        # the default_player settings
        if self.settings.default_player_name is not None:
            data = {
                "default_player": self.settings.default_player_name
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "default_player status not defined"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_default_player(self):
        """
        Set the Brain.ai Core default_player value

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"default_player": "myPlayer"}' http://127.0.0.1:5000/settings/default_player
        """

        if not request.get_json() or 'default_player' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'default_player' not set"
            }
            return jsonify(error=data), 400
        # get if present
        value = self.get_value_flag_from_request(http_request=request,
                                                 flag_to_find="default_player",
                                                 is_boolean=False)

        SettingEditor.set_default_player(default_player_name=value)

        data = {
            "default_player": self.settings.default_player_name
        }
        return jsonify(data), 200

    @requires_auth
    def get_default_trigger(self):
        """
        Return the current default_trigger value from settings

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings/default_trigger
        """

        # the default_trigger settings
        if self.settings.default_trigger_name is not None:
            data = {
                "default_trigger": self.settings.default_trigger_name
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "default_trigger status not defined"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_default_trigger(self):
        """
        Set the Brain.ai Core default_trigger value

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"default_trigger": "myTrigger"}' http://127.0.0.1:5000/settings/default_trigger
        """

        if not request.get_json() or 'default_trigger' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'default_trigger' not set"
            }
            return jsonify(error=data), 400

        # get if present
        value = self.get_value_flag_from_request(http_request=request,
                                                 flag_to_find="default_trigger",
                                                 is_boolean=False)

        SettingEditor.set_default_trigger(default_trigger=value)

        data = {
            "default_trigger": self.settings.default_trigger_name
        }
        return jsonify(data), 200

    @requires_auth
    def get_hooks(self):
        """
        Return the list of hooks from settings

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings/hooks
        """

        if self.settings.hooks is not None:
            data = {
                "hooks": self.settings.hooks
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "hooks are not defined"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_hooks(self):
        """
        Set the Brain.ai Core hooks value

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"hook_name":"hook_value","hooke_name2":"hook_value2"}' http://127.0.0.1:5000/settings/hooks
        """

        if not request.get_json():
            data = {
                "Error": "No Parameters not"
            }
            return jsonify(error=data), 400

        # get if present
        value = request.get_json()

        if not isinstance(value, dict):
            data = {
                "Error": "Hooks must be a dictionary"
            }
            return jsonify(error=data), 400

        SettingEditor.set_hooks(hooks=value)

        data = {
            "hooks": self.settings.hooks
        }
        return jsonify(data), 200

    @requires_auth
    def get_variables(self):
        """
        Return the list of variables from settings

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings/variables
        """

        if self.settings.variables is not None:
            data = {
                "variables": self.settings.variables
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "variables are not defined"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_variables(self):
        """
        Set the Brain.ai Core variables value.
        Can be used with a dictionary of variables :
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"variable1":"variables_value","variables_name2":"variables_value2"}' http://127.0.0.1:5000/settings/varaibles
        """

        if not request.get_json():
            data = {
                "Error": "No Parameters provided"
            }
            return jsonify(error=data), 400

        # get if present
        value = request.get_json()

        if not isinstance(value, dict):
            data = {
                "Error": "Variables must be a dictionary"
            }
            return jsonify(error=data), 400

        SettingEditor.set_variables(variables=value)

        data = {
            "variables": self.settings.variables
        }
        return jsonify(data), 200

    def audio_analyser_callback(self, order):
        """
        Callback of the OrderListener. Called after the processing of the audio file
        This method will
        - call the Order Analyser to analyse the  order and launch corresponding synapse as usual.
        - get a list of launched synapse.
        - give the list to the main process via self.launched_synapses
        - notify that the processing is over via order_analyser_return
        :param order: string order to analyse
        :return:
        """
        logger.debug("[FlaskAPI] audio_analyser_callback: order to process -> %s" % order)
        api_response = SynapseLauncher.run_matching_synapse_from_order(order,
                                                                       self.brain,
                                                                       self.settings,
                                                                       is_api_call=True)
        self.api_response = api_response

        # this boolean will notify the main process that the order have been processed
        self.order_analyser_return = True

    def get_value_flag_from_request(self, http_request, flag_to_find, is_boolean=False):
        """
        Get the value flag from the request if exist, None otherwise !
        :param http_request:
        :param flag_to_find: json flag to find in the http_request
        :param is_boolean: True if the expected value is a boolean, False Otherwise.
        :return: the Value of the flag that has been found in the request
        """
        flag_value = None
        try:
            received_json = http_request.get_json(force=True, silent=True, cache=True)
            if flag_to_find in received_json:
                flag_value = received_json[flag_to_find]
                if is_boolean:
                    flag_value = Utils.str_to_bool(flag_value)
        except TypeError:
            # no json received
            pass
        logger.debug("[FlaskAPI] Value found for : type %s,  %s : %s" % (flag_to_find, type(flag_value), flag_value))
        return flag_value

    @staticmethod
    def get_parameters_from_request(http_request):
        """
        Get "parameters" object from the
        :param http_request:
        :return:
        """
        parameters = None
        try:
            # Silent True in case no parameters it does not raise an error
            received_json = http_request.get_json(silent=True, force=True)
            if 'parameters' in received_json:
                parameters = received_json['parameters']
        except TypeError:
            pass
        logger.debug("[FlaskAPI] Overridden parameters: %s" % parameters)

        return parameters

    def get_current_settings(self):
        """
        get the current settings config
        test with curl:
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings
        """
        logger.debug("[FlaskAPI] get_current_settings: all")
        data = jsonify(settings=self.settings.serialize())
        return data, 200



#! /usr/bin/env python
# -*- coding: utf-8 -*-
import re
import sys
from setuptools import setup, find_packages
from codecs import open
from os import path

basedir = path.abspath(path.dirname(__file__))

# locate our version number
def read_version_py(file_name):
    try:
        version_string_line = open(file_name, "rt").read()
    except EnvironmentError:
        return None
    else:
        version_regex = r"^version_str = ['\"]([^'\"]*)['\"]"
        mo = re.search(version_regex, version_string_line, re.M)
        if mo:
            return mo.group(1)

VERSION_PY_FILENAME = 'brain/_version.py'
version = read_version_py(VERSION_PY_FILENAME)

setup(
    name='brain-ai',
    version=version,
    description='Build your own intelligent personal assistant.',
    long_description='Brain.ai is an open source intelligent personal assistant development package.',
    url='https://github.com/brain-ai/brain',
    author='Alexander Paul P. Quinit',
    author_email='paulquinit@gmail.com',
    license='GNU General Public License v3 (GPLv3)',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Home Automation',
        'Topic :: Multimedia :: Sound/Audio :: Speech',
        'Topic :: Multimedia :: Sound/Audio :: Sound Synthesis',
        'Topic :: Scientific/Engineering :: Artificial Intelligence'
    ],
    keywords='assistant bot TTS STT brain',

    packages=find_packages(exclude=['docs', 'tests', 'brain.py', 'brain/brain.yml', 'brain/settings.yml']),

    install_requires=[
        'pyyaml>=3.12',
        'six>=1.10.0',
        'SpeechRecognition>=3.7.1',
        'markupsafe>=1.0',
        'pyaudio>=0.2.11',
        'pyasn1>=0.2.3',
        'ansible>=2.3,<2.4',
        'jinja2>=2.10',
        'cffi>=1.9.1',
        'ipaddress>=1.0.17',
        'flask>=0.12',
        'Flask-Restful>=0.3.5',
        'flask_cors>=3.0.2',
        'requests>=2.13',
        'httpretty>=0.8.14',
        'mock>=2.0.0',
        'Flask-Testing>=0.6.2',
        'apscheduler>=3.3.1',
        'GitPython>=2.1.3',
        'packaging>=16.8',
        'transitions>=0.4.3',
        'sounddevice>=0.3.7',
        'SoundFile>=0.9.0',
        'pyalsaaudio>=0.8.4',
        'sox>=1.3.0',
        'paho-mqtt>=1.3.0',
        'voicerss_tts>=1.0.3'
    ],

    package_data={
        'brain': [
            'trigger/snowboy/armv7l/python27/_snowboydetect.so',
            'trigger/snowboy/x86_64/python27/_snowboydetect.so',
            'trigger/snowboy/x86_64/python34/_snowboydetect.so',
            'trigger/snowboy/x86_64/python35/_snowboydetect.so',
            'trigger/snowboy/x86_64/python36/_snowboydetect.so',
            'trigger/snowboy/resources/*',
            'sounds/*'
         ],
    },

    entry_points={
        'console_scripts': [
            'brain=brain:main',
        ],
    },
)
